# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

class CommandError(Exception):
    pass

class DependencyError(Exception):
    pass

class SwarmError(Exception):
    pass
